clc;
clear;
close all;

CostFunction=@Sphere;
nVar=10; 
VarSize=[1 nVar]; 
VarMin=-1;
VarMax=1;
VarRange=[VarMin,VarMax];

MaxIt=700;
nPop=300; 
pCrossover=0.78; 
nCrossover=round(pCrossover*nPop/2)*2; 
pMutation=0.38;
nMutation=round(pMutation*nPop); 
SelectionPressure=12; 

individual.Position=[];
individual.Cost=[];

pop=repmat(individual,nPop,1);

for i=1:nPop 
    pop(i).Position=unifrnd(VarMin,VarMax,VarSize);
    pop(i).Cost=CostFunction(pop(i).Position);
end

Costs=[pop.Cost];
[Costs SortOrder]=sort(Costs);
pop=pop(SortOrder);
WorstCost=Costs(end);

BestSol=[]; 
BestCost=zeros(MaxIt,1);
MeanCost=zeros(MaxIt,1);

for it=1:MaxIt
    Costs=[pop.Cost];
    P=exp(-SelectionPressure*Costs/WorstCost);
    P=P/sum(P);
    C=cumsum(P);
    pop2=repmat(individual,nCrossover/2,2);
    for k=1:nCrossover/2
        i1=find(rand<=C,1,'first'); 
        i2=find(rand<=C,1,'first'); 
        p1=pop(i1);
        p2=pop(i2);
        [ch1 ch2]=Crossover(p1,p2);
        ch1.Cost=CostFunction(ch1.Position);
        ch2.Cost=CostFunction(ch2.Position);
        pop2(k,1)=ch1;
        pop2(k,2)=ch2;
    end
    pop2=pop2(:);
    
    pop3=repmat(individual,nMutation,1);
    for k=1:nMutation
        i=find(rand<=C,1,'first'); 
        q=Mutate(pop(i),VarRange);
        q.Cost=CostFunction(q.Position);
        pop3(k)=q;
    end
    
    pop=[pop
         pop2
         pop3];  
    
    Costs=[pop.Cost];
    [Costs SortOrder]=sort(Costs);
    pop=pop(SortOrder);
    WorstCost=max(WorstCost,Costs(end));
    
    pop=pop(1:nPop);
    Costs=Costs(1:nPop);
    
    BestSol=pop(1);
    BestCost(it)=Costs(1);
    MeanCost(it)=mean(Costs);
    
    disp(['Iteration ' num2str(it) ':   ' ...
          'Best Cost = ' num2str(BestCost(it)) '  , ' ...
          'Mean Cost = ' num2str(MeanCost(it))]);
end

figure;
semilogy(BestCost,'r','LineWidth',2);
hold on;
semilogy(MeanCost,'b:','LineWidth',2);
xlabel('Iteration (Generation)');
ylabel('Cost');
legend('Best Cost','Mean Cost');